<?php
/**
* @version $Id: $
* @package osm-things
* @copyright Copyright (C) 2008 Sebastian Klemm.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

define( '_VALID_OSM', TRUE );
define( '_PATH', './' );
$DEBUG = FALSE;
if($DEBUG) error_reporting(E_ALL);

session_start();

include("./config.inc.php");
include("./libraries/functions.inc.php");
include("./libraries/classes.php");
include("./libraries/html.classes.php");
include("./libraries/GeoCalc.class.php");
include("./traces.html.php");

setlocale (LC_TIME, $cfg['config_locale']);
include("./languages/".get_lang($cfg['config_language']).".php");
include("./head.html.php");

if($cfg['show_exec_time'])
    $startTime = microtime_float();

if($DEBUG) {
    foreach($_POST as $akey => $val)
        out("<b>$akey</b> = $val", "OUT_DEBUG");
}

$p = intval($_GET['p']);
$id = intval($_GET['id']);
$task = strip_tags($_GET['task']);
$option = strip_tags($_GET['option']);
$f_date = strip_tags($_GET['date']);
$f_lat = floatval($_GET['lat']);
$f_lon = floatval($_GET['lon']);
$f_lat_range = floatval($_GET['lat_range']);
$f_lon_range = floatval($_GET['lon_range']);
$description = strip_tags($_GET['description']);
$submit = strip_tags($_POST['submit']);
$confirm = strip_tags($_POST['confirm']);

// convert lat/lon from float value to database integer format
if($option == "search") {
	$option = "filter";
	$f_lat *= 1000000; 
	$f_lon *= 1000000; 
	$f_lat_range *= 1000000; 
	$f_lon_range *= 1000000; 
}

// connect to database
$link = db_connect_h($cfg['db_host'], $cfg['db_name'], $cfg['db_user'], $cfg['db_password']);

HTML::heading(_APP_NAME, 2);
HTML::main_menu();

switch ($task) {
	case 'gpx':
		viewGPX($p);
		break;

	case 'details':
		details($id);
		break;

	case 'image':
		makeImage($id);
		break;

	case 'view':
		viewTrace($id, $p);
		break;

	case 'upload':
		uploadTrace();
		break;

	case 'import':
		importTrace();
		break;

	case 'export':
		exportTrace($id);
		break;

	case 'download':
		downloadTrace($id);
		break;

	case 'delete':
		deleteTrace($id, $confirm);
		break;

	case 'searchGPX':
		searchGPX();
		break;

	case 'searchTP':
		searchTP();
		break;

	default:
        viewTrace(NULL, NULL);
		break;
}


function viewGPX($page) {
	global $DEBUG, $cfg;
	global $option, $description;
    HTML::heading(_MENU_GPX_VIEW, 3);
	$search_url = "";
	$limit = $cfg['result_table_limit'];
    $query = "SELECT SQL_CALC_FOUND_ROWS * FROM `${cfg['db_table_prefix']}gpx_files` ";
	if($option == "filter") {
		$query .= "WHERE `description` LIKE '%$description%' ";
		$search_url = "&option=filter&description=$description";
	}
	$query .= "ORDER BY `id` ASC LIMIT ". ($page*$limit) .", $limit ;";
    $result = db_query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysql_num_rows($result)) {
		$num_found = mysql_result(db_query("SELECT FOUND_ROWS();") ,0);
		HTML::viewPagination($page, ceil($num_found/$limit), 
			'traces.php?task=gpx'.$search_url);
		HTML_traces::viewGpxTableHeader();
		while($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
			HTML_traces::viewGpxTableRow($row);
		}
		HTML_traces::viewGpxTableFooter();
		HTML::viewPagination($page, ceil($num_found/$limit), 
			'traces.php?task=gpx'.$search_url);
		HTML::message($num_found ._DB_GPX_AVAILABLE);
	}
	else
		HTML::message(_TRC_NO_GPX_IN_DB);
}

function makeImage($id) {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_IMAGE, 3);
    HTML::message(_CMN_NOT_IMPLEMENTED);
    // 1 pixel image for generating image file
	echo '<img src="./image.php?id='.$id.'&name=gpx_'.$id.
		'&width='.$cfg['image_size'].'&height='.$cfg['image_size'].'" '.
		'width="1" height="1" border="1" alt="GPX-Trace" />';
	// real size image and link to big image
	echo '<a href="./image.php?id='.$id.
		'&width='.$cfg['image_big_size'].'&height='.$cfg['image_big_size'].'">
		<img src="./image.php?id='.$id.
		'&width='.$cfg['image_size'].'&height='.$cfg['image_size'].'" '.
		'width="'.$cfg['image_size'].'" height="'.$cfg['image_size'].'" '.
		'border="1" alt="GPX-Trace" /></a>';
}

function details($id) {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_DETAILS, 3);
    HTML::message(_TRC_DETAILS_OF_GPX . $id);
    $query = "SELECT * FROM `${cfg['db_table_prefix']}gpx_files` WHERE `id` = '$id' ;";
    $result = db_query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysql_num_rows($result)) {
    	while($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
			HTML_traces::viewGpxDetails($row);
    	}
    }else {
		HTML::message(_TRC_GPX_DOES_NOT_EXIST);
		return 1;
    }
    $query = "SELECT `gpx_id`, 
		MIN(`latitude`) AS 'minlat', MAX(`latitude`) AS 'maxlat', AVG(`latitude`) AS 'avglat', 
		MIN(`longitude`) AS 'minlon', MAX(`longitude`) AS 'maxlon', AVG(`longitude`) AS 'avglon',
		MIN(`altitude`) AS 'minalt', MAX(`altitude`) AS 'maxalt', AVG(`altitude`) AS 'avgalt', 
		MIN(`timestamp`) AS 'mints', MAX(`timestamp`) AS 'maxts',
		MIN(`fix`) AS 'minfix', MAX(`fix`) AS 'maxfix', AVG(`fix`) AS 'avgfix', 
		MIN(`sat`) AS 'minsat', MAX(`sat`) AS 'maxsat', AVG(`sat`) AS 'avgsat', 
		MIN(`hdop`) AS 'minhdop', MAX(`hdop`) AS 'maxhdop', AVG(`hdop`) AS 'avghdop', 
		MIN(`pdop`) AS 'minpdop', MAX(`pdop`) AS 'maxpdop', AVG(`pdop`) AS 'avgpdop'
		FROM `${cfg['db_table_prefix']}gpx_import` WHERE `gpx_id` = '$id' GROUP BY `gpx_id` ;";
    $result = db_query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysql_num_rows($result)) {
    	while($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
			HTML_traces::viewGpxDetailsTable($row);
			$trkptStats = $row;
    	}
	}

	// TEST CODE for distance calculation
	# TO DO: fix NaN bug with id=37
	$stw = microtime_float();
	$query = "SELECT latitude, longitude, timestamp 
		FROM `${cfg['db_table_prefix']}gpx_import` WHERE `gpx_id` = '$id' ;";
	$result = db_query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
	
	if(mysql_num_rows($result)) {
		#$DEBUG = TRUE;
		$pt = array('i'=>0, 'x'=>0, 'y'=>0, 'xold'=>0, 'yold'=>0, 't'=>0, 'told'=>0);
		$distances[] = array(0, 0, strtotime($trkptStats['mints']));
	    $oGC = new GeoCalc();
		
		while($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
			$pt['i']++;
			$pt['told'] = $pt['t'];
			$pt['xold'] = $pt['x'];
			$pt['yold'] = $pt['y'];
			$pt['y'] = $row['latitude'] /1000000;
			$pt['x'] = $row['longitude'] /1000000;
			$pt['t'] = strtotime($row['timestamp']);
			 
			if($pt['xold'] && $pt['yold']) {
				$distP2P = $oGC->GCDistance($pt['y'], $pt['x'], $pt['yold'], $pt['xold']);
				// illegal distance value
				if(!$distP2P || is_nan($distP2P)) {
					if($DEBUG) {
						out($row['timestamp']." :  xo=".$pt['xold']." yo=".$pt['yold'].
							" x=".$pt['x']." y=".$pt['y']." d=".$distP2P, 'OUT_DEBUG');
					}
					continue;
				}
				// distance between 2 trackpoints greater than threshold
				if($distP2P > $cfg['dist_threshold']) {
					if($DEBUG)
						out("dist to big (suspect): ".$distP2P, 'OUT_DEBUG');
				}
				// time between 2 trackpoints greater than threshold
				elseif(($pt['t'] - $pt['told']) > $cfg['time_threshold']) {
					$distances[] = array($dist, $pt['told'], $pt['t']);
					$dist = 0;
				}
				// sum point-to-point distances
				else {
					$dist += $distP2P;
				}
			}
		}
		$distances[] = array($dist, strtotime($trkptStats['maxts']), 0);
		HTML_traces::viewGpxDistances($distances);
		if($DEBUG)
			out("(dist calc took ". round(microtime_float() - $stw, 4) ." s)", 'OUT_DEBUG');
		/*
		// TEST CODE for speed calculation (moved to traces.html.php)
		$track_time = strtotime($trkptStats['maxts']) - strtotime($trkptStats['mints']);
		out("total: ". $track_time ." s", 'OUT_DEBUG');
		foreach($distances as $key=>$val) {
			$track_time += $val[1] - $val[2];
		}
		out("trip time: ". $track_time ." s", 'OUT_DEBUG');
		$speed = round(array_sum_multidim($distances, NULL, 0) / $track_time * 3600, 2);
		out("average speed: $speed km/h", 'OUT_DEBUG');
		*/
	}
}

function viewTrace($id, $page) {
	global $DEBUG, $cfg;
	global $option, $f_date, $f_lat, $f_lon, $f_lat_range, $f_lon_range;
	HTML::heading(_MENU_TRKPT_VIEW, 3);
	$search_url = "";
	$limit = $cfg['result_table_limit'];
	$query = "SELECT SQL_CALC_FOUND_ROWS * FROM `${cfg['db_table_prefix']}gpx_import` ";
	if($id)
		$query .= "WHERE `gpx_id` = '$id' ";
	if($option == "filter") {
		$query .= "WHERE 1 ";
		if($f_date)
			$query .= "AND DATE(`timestamp`) = '$f_date' ";
		if($f_lat)
			$query .= "AND `latitude` BETWEEN '$f_lat'-'$f_lat_range' 
					AND '$f_lat'+'$f_lat_range' ";
		if($f_lon)
			$query .= "AND `longitude` BETWEEN '$f_lon'-'$f_lon_range' 
					AND '$f_lon'+'$f_lon_range' ";

		$search_url = "&option=filter&date=$f_date&lat=$f_lat&lon=$f_lon&lat_range=$f_lat_range&lon_range=$f_lon_range";
	}
	$query .= "ORDER BY `timestamp` LIMIT ". ($page*$limit) .", $limit ;";
    $result = db_query($query);
	if($DEBUG)	out($query, 'OUT_DEBUG');
    if(mysql_num_rows($result)) {
		$num_found = mysql_result(db_query("SELECT FOUND_ROWS();") ,0);
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"traces.php?task=view&id=$id".$search_url);
		HTML_traces::viewTraceTableHeader();
		while($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
			HTML_traces::viewTraceTableRow($row);
		}
		HTML_traces::viewTraceTableFooter();
		HTML::viewPagination($page, ceil($num_found/$limit), 
			"traces.php?task=view&id=$id".$search_url);
		HTML::message($num_found ._DB_TRKPTS_AVAILABLE);
		if($option == "filter") {
			HTML::message("<a href='export.php?name=trackpoints$search_url'>". _TRC_EXPORT_AS_GPX ."</a>");
		}
	}
	else
		HTML::message(_TRC_NO_TRKPTS_IN_DB);
}

function uploadTrace() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_UPL, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
    	HTML_traces::fileUploadForm('traces.php?task=import', $cfg['max_file_size']);
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function importTrace() {
	global $DEBUG, $cfg;
	set_time_limit(10);
    HTML::heading(_MENU_GPX_IMPORT, 3);
	if($DEBUG) {
	    out($_FILES['userfile']['name'] ." (file name)", 'OUT_DEBUG');
	    out($_FILES['userfile']['size'] ." Bytes (size)", 'OUT_DEBUG');
	    out($_FILES['userfile']['error'] ." (error number)", 'OUT_DEBUG');
	}
	
	$uploaddir = 'files/';
	$uploadfile = $uploaddir. $_FILES['userfile']['name'];
    #if($_FILES['userfile']['error'])
    if(!move_uploaded_file($_FILES['userfile']['tmp_name'], $uploadfile))
    	echo _TRC_UPL_ERROR . $_FILES['userfile']['error'];
    else {
    	HTML::message(_TRC_UPL_SUCCESS);
		
    	$trkpt_number = 0;
    	$line_number = 0;
    	#$file_handle = fopen($_FILES['userfile']['tmp_name'] ,'r');
    	$file_handle = fopen("files/".$_FILES['userfile']['name'] ,'r');
    	if($file_handle) {
    		// insert gpx into database
	        $query = "INSERT IGNORE INTO `${cfg['db_table_prefix']}gpx_files` SET 
	        	`name` = '".$_FILES['userfile']['name']."', 
	        	`size` = '".$_FILES['userfile']['size']."',
	        	`latitude` = '0', 
	        	`longitude` = '0',
	        	`timestamp` = NOW(),
	        	`public` = '1',
	        	`description` = '".$_POST['description']."',
	        	`inserted` = '0' ;";
	        $result = db_query($query);
	        $gpx_id = mysql_insert_id();
	        
			// read all lines from file			
			while (!feof($file_handle)) {
				// skip lines w/o track point data
				do {				
					$buffer = fgets($file_handle, 128);					
					#if($DEBUG)	out($line_number.": ".htmlentities($buffer), 'OUT_DEBUG');
					$line_number ++;
				} while ($buffer && strpos($buffer, "<trkpt") === false);
				
				$trkpt = new TrackPoint;

				// extract track point data
				while ($buffer && strpos($buffer, "</trkpt") === false) {
					// extract data from line
					preg_match("/[^<|\s][^>|\s]+/", $buffer, $hit);
					
					switch ($hit[0]) {
					case 'trkpt':
						// lat / lon
						preg_match('/lat="(.+?)"/', $buffer, $hitLat);
						preg_match('/lon="(.+?)"/', $buffer, $hitLon);
						#out("___LAT: $hitLat[1] ;___LON: $hitLon[1]", 'OUT_DEBUG');
						#print_r($hitLat); print_r($hitLon);
						$trkpt->setIntPos($hitLat[1], $hitLon[1]);
						break;				
					case 'fix':
						preg_match("/\d+/", $buffer, $hit);
						#out("___FIX: $hit[0]", 'OUT_DEBUG');
						$trkpt->setFix($hit[0]);
						break;				
					case 'sat':
						preg_match("/\d+/", $buffer, $hit);
						#out("___SAT: $hit[0]", 'OUT_DEBUG');
						$trkpt->setSat($hit[0]);
						break;				
					case 'hdop':
						preg_match("/\d+[.|,]?\d+/", $buffer, $hit);
						#out("___HDOP: $hit[0]", 'OUT_DEBUG');
						$trkpt->setHDOP($hit[0]);
						break;				
					case 'pdop':
						preg_match("/\d+[.|,]?\d+/", $buffer, $hit);
						#out("___PDOP: $hit[0]", 'OUT_DEBUG');
						$trkpt->setPDOP($hit[0]);
						break;				
					case 'ele':
						preg_match("/\d+[.|,]?\d+/", $buffer, $hit);
						#out("___ELE: $hit[0]", 'OUT_DEBUG');
						$trkpt->setEle($hit[0]);
						break;				
					case 'course':
						preg_match("/\d+[.|,]?\d+/", $buffer, $hit);
						#out("___COURSE: $hit[0]", 'OUT_DEBUG');
						$trkpt->setCourse($hit[0]);
						break;				
					case 'speed':
						preg_match("/\d+[.|,]?\d+/", $buffer, $hit);
						#out("___SPEED: $hit[0]", 'OUT_DEBUG');
						$trkpt->setSpeed($hit[0]);
						break;				
					case 'time':
						preg_match("/([\d-]+)T([\d:]+)/", $buffer, $hit);
						#out("___TIME: $hit[1]_ $hit[2]", 'OUT_DEBUG');
						#print_r($hit);
						$trkpt->setTime($hit[1]." ".$hit[2]);
						break;				
					}
					if ($buffer) {					
						$buffer = fgets($file_handle, 128);
						#if($DEBUG)	out($line_number.": ".htmlentities($buffer), 'OUT_DEBUG');
						$line_number ++;
					}
				}
				$trkpt_number ++;
				#print_r($trkpt);
				
				// insert track point into data base
				$trkpt->insert($gpx_id);
			}
			fclose($file_handle);
			#if($DEBUG)	out("Last line #: ".$line_number, 'OUT_DEBUG');
			#echo $trkpt_number . _TRC_TRKPTS_PROCESSED;
			HTML::message($trkpt_number . _TRC_TRKPTS_PROCESSED);
			HTML::message("<a href='traces.php?task=details&id=". 
				$gpx_id ."'>". _MENU_GPX_DETAILS ."</a>");
		}else
			HTML::message(_TRC_FILE_ERROR);
    }
}

function exportTrace($id) {
	global $DEBUG, $cfg, $option;
    HTML::heading(_MENU_GPX_EXPORT, 3);
    HTML::message(_CMN_NOT_IMPLEMENTED);
}

function downloadTrace($id) {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_DOWNL, 3);
    HTML::message(_CMN_NOT_IMPLEMENTED);
}

function deleteTrace($id, $confirm) {
	global $DEBUG, $cfg;
	global $submit;
    HTML::heading(_MENU_GPX_DELETE, 3);
    if(!$cfg['public_host'] || check_password($cfg['admin_password'])) {
	    if(!$submit) {
	    	HTML_traces::deleteForm('traces.php?task=delete&id='.$id, $id);
	    } else {
			if($confirm == _CMN_YES) {
				// delete track points
		        $query = "DELETE IGNORE FROM `${cfg['db_table_prefix']}gpx_import` 
		        	WHERE `gpx_id` = '$id' ;";
		        $result = db_query($query);
		        $num_rows = mysql_affected_rows();
		        HTML::message_r(_TRC_TRKPT_DELETED, $num_rows);
	
				// delete gpx file
		        $query = "DELETE IGNORE FROM `${cfg['db_table_prefix']}gpx_files` 
		        	WHERE `id` = '$id' ;";
		        $result = db_query($query);
		        $num_rows = mysql_affected_rows();
		        HTML::message_r(_TRC_GPX_DELETED, $num_rows);
			} else {
				HTML::message(_TRC_NO_CONFIRM_DELETE);
			}    
	    }
    }else {
    	HTML::message(_NOT_AUTH ." ". _DO_LOGIN);
    }
}

function searchGPX() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_GPX_SEARCH, 3);
    HTML_traces::searchGpxForm('');
}

function searchTP() {
	global $DEBUG, $cfg;
    HTML::heading(_MENU_TRKPT_SEARCH, 3);
    HTML_traces::searchTrkPtForm('');
}


if($cfg['show_exec_time']) {
    $endTime = microtime_float();
    $exectime = round($endTime - $startTime, 4);
}
include("./foot.html.php");
?>